connect /
set echo on
/*
drop table stage;
create table stage as select object_name from all_objects;
*/
clear screen
create or replace package encryption_wrapper
as    
    -- using varchar2 and utl_i18n
    function encrypt1( p_string in varchar2,
                       p_key    in varchar2 ) 
    return raw;

    function decrypt1( p_raw in raw,
                       p_key in varchar2 )
    return varchar2;

    -- using blobs and clobs
    function encrypt2( p_string in varchar2,
                       p_key    in varchar2 ) 
    return raw;

    function decrypt2( p_raw in raw,
                       p_key in varchar2 )
    return varchar2;
end;
/
pause

set termout off
create or replace package body encryption_wrapper
as
   g_encrypt_typ     constant PLS_INTEGER default
                     DBMS_CRYPTO.ENCRYPT_AES256 
                     + DBMS_CRYPTO.CHAIN_CBC 
                     + DBMS_CRYPTO.PAD_PKCS5;
    g_clob clob default 'x';
    g_blob blob default hextoraw( '00' );

function padkey( p_key in varchar2 ) return raw
is
begin
    return utl_raw.cast_to_raw(rpad(p_key,32));
end;

function encrypt1( p_string in varchar2,
                   p_key    in varchar2 ) 
return raw
is
begin
   return DBMS_CRYPTO.ENCRYPT
          ( src => UTL_I18N.STRING_TO_RAW (p_string,  'AL32UTF8'),
            typ => g_encrypt_typ,
            key => padkey( p_key ) );
end encrypt1;

function decrypt1( p_raw in raw,
                   p_key in varchar2 )
return varchar2
is
begin
    return utl_i18n.raw_to_char( 
              dbms_crypto.decrypt
              ( src => p_raw,
                typ => g_encrypt_typ,
                key => padkey(p_key) ), 
              'AL32UTF8' );
end;


function encrypt2( p_string in varchar2,
                   p_key    in varchar2 ) 
return raw
is
begin    
    g_clob := p_string;
    dbms_crypto.encrypt
    ( dst => g_blob, 
      src => g_clob, 
      typ => g_encrypt_typ, 
      key => padkey( p_key ) );

    return g_blob;
end encrypt2;


function decrypt2( p_raw in raw,
                   p_key in varchar2 )
return varchar2
is
begin
    g_blob := p_raw;
    dbms_crypto.decrypt( dst => g_clob, 
                         src => g_blob, 
                         typ => g_encrypt_typ,
                         key => padkey( p_key ) );
    return g_clob;
end decrypt2;

end;
/
set termout on

clear screen
select text from user_source where name = 'ENCRYPTION_WRAPPER' 
and type = 'PACKAGE BODY' and line between 1 and 15 order by line;
pause
clear screen
select text from user_source where name = 'ENCRYPTION_WRAPPER' 
and type = 'PACKAGE BODY' and line between 16 and 25 order by line;
pause
clear screen
select text from user_source where name = 'ENCRYPTION_WRAPPER' 
and type = 'PACKAGE BODY' and line between 27 and 38 order by line;
pause
clear screen
select text from user_source where name = 'ENCRYPTION_WRAPPER' 
and type = 'PACKAGE BODY' and line between 41 and 54 order by line;
pause
clear screen
select text from user_source where name = 'ENCRYPTION_WRAPPER' 
and type = 'PACKAGE BODY' and line between 57 and 70 order by line;
pause

drop table t;
clear screen
create table t 
( orig_string varchar2(30), 
  encrypt1    raw(32),
  encrypt2    raw(32),
  decrypt1    varchar2(30),
  decrypt2    varchar2(30)
)
/
insert into t (orig_string) values ( 'hello world' );
commit;
pause

clear screen
update t 
   set encrypt1 = 
        encryption_wrapper.encrypt1( orig_string, 'shh, it is a secret'),
       encrypt2 = 
        encryption_wrapper.encrypt2( orig_string, 'shh, it is a secret')
/
pause

clear screen
update t
   set decrypt1 = 
         encryption_wrapper.decrypt1( encrypt1, 'shh, it is a secret' ),
       decrypt2 = 
         encryption_wrapper.decrypt2( encrypt2, 'shh, it is a secret' )
/
pause
set termout off
@printtbl
set termout on
clear screen
exec print_table( 'select * from t' );
pause

drop table t;
clear screen
create table t
( last_name      varchar2(30),
  encrypted_name raw(32),
  decrypted_name varchar2(30)
)
pctfree 70 
/
insert into t(last_name) 
select object_name 
from stage 
where rownum <= 20000;
pause

clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
begin
    update t set encrypted_name = 
        encryption_wrapper.encrypt1(last_name,'Secret Key Secret Key Secret Key');
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause

clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
begin
    update t set encrypted_name = 
        encryption_wrapper.encrypt2(last_name,'Secret Key Secret Key Secret Key');
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause


clear screen
declare
    l_start number;
begin
    execute immediate 'truncate table t';
    l_start := dbms_utility.get_cpu_time;
    insert into t (last_name,encrypted_name) 
	select object_name, null from stage
	where rownum <= 20000;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause

clear screen
declare
    l_start number;
begin
    execute immediate 'truncate table t';
    l_start := dbms_utility.get_cpu_time;
    insert into t (last_name,encrypted_name) 
    select object_name, 
	       encryption_wrapper.encrypt1
		   (object_name,'Secret Key Secret Key Secret Key')
      from stage
	  where rownum <= 20000;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause
clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
begin
    for x in (select last_name, last_name xx from t)
    loop
        if (nvl(x.last_name,'x') != nvl(x.xx,'x') )
        then
            raise program_error;
        end if;
    end loop;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause
clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
begin
    for x in (select last_name, 
                     encryption_wrapper.decrypt1
                     (encrypted_name,'Secret Key Secret Key Secret Key') xx
                from t)
    loop
        if (nvl(x.last_name,'x') != nvl(x.xx,'x') )
        then
            raise program_error;
        end if;
    end loop;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause
clear screen
declare
    l_start number := dbms_utility.get_cpu_time;
begin
    for x in (select last_name,
                     encryption_wrapper.decrypt2
                     (encrypted_name,'Secret Key Secret Key Secret Key') xx
                from t)
    loop
        if (nvl(x.last_name,'x') != nvl(x.xx,'x') )
        then
            raise program_error;
        end if;
    end loop;
    dbms_output.put_line( (dbms_utility.get_cpu_time-l_start) || ' hsecs' );
end;
/
/
pause
